// createWorld code snippet courtesy of:
//  https://stackoverflow.com/questions/45903238/es6-way-to-create-an-array-of-arrays-with-random-numbers#
//

// total dimensions of the new world (lat=width; long=height)
const lat = 11;
const long = 11;

// world terrain consists of land (continents) and water (oceans)
const land = 'M';
const ocean = 'o';

// create a world consisting of randomly generated continents ('M') and oceans('o')
const createWorld = ({width = lat, height = long}) =>
    Array.from ({ length: height },
        () => Array.from({ length: width },
            (mass=['M','o']) => mass[Math.floor(Math.random() * 2)],
        )
    )
    
// generate an unique new world of size lat (x width) and long (x height)
const newWorld = createWorld(lat, long);
console.log(newWorld);
console.log(newWorld.join('\n') + '\n\n');

// randomly select a start position (lat and long) from which to view the world
// determine the type of terrain (land or water) of the positionInWorld
// ensure that start position is land
const positionInWorld = function() {
    let x, y, location;
    do {
        x = Math.floor(Math.random() * lat);  // width
        y = Math.floor(Math.random() * long); // height 
        location = newWorld[x][y];   
    }
    while (location === ocean);
    console.log('You are currently standing at position: latitude ' + x + ' and longitude ' + y);
    console.log('you are currently standing on ' + location + ', land');
    return {x, y};
};

// assign return value's of positionInWorld() to x, y
let {x, y} = positionInWorld();


// ensure that continentSize counter stays within lat (world width) range
const inLat = (w) => {return w > -1 && w < lat}

// ensure that continentSize counter stays within long (world height) range
const inLong = (h) => {return h > -1 && h < long}



// from positionInWorld, count number of adjacent land masses, 'M', 
// in order to determine size of continent at positionInWorld
function continentSize(x, y) {
    if (newWorld[x][y] != land) {
        return 0;
    }
    // the land currently standing on 
    let mass = 1;
    
    newWorld[x][y] = 'C'; // C refers to land that has been counted
    // row above
    mass = mass + continentSize(inLat(x-1) ? x-1 : x, y);
    // row diagonal upper right
    mass = mass + continentSize(inLat(x-1) ? x-1 : x, inLong(y+1) ? y+1 : y);
    // row right
    mass = mass + continentSize(x, inLong(y+1) ? y+1 : y);
    // row diagonal bottom right
    mass = mass + continentSize(inLat(x+1) ? x+1 : x, inLong(y+1) ? y+1 : y);
    // row bottom
    mass = mass + continentSize(inLat(x+1) ? x+1 : x, y);
    // row diagonal bottom left
    mass = mass + continentSize(inLat(x+1) ? x+1 : x, inLong(y-1) ? y-1 : y);
    // row left 
    mass = mass + continentSize(x, inLong(y-1) ? y-1 : y);
    // row diagonal top left
    mass = mass + continentSize(inLat(x-1) ? x-1 : x, inLong(y-1) ? y-1 : y);

    // console.log(mass);
    return mass
};



const totalMass = continentSize(x, y);
console.log('The total size of the continent you are standing on is ' + totalMass + ' out of ' + (lat * long))

console.log('The world displaying the continent that has been counted'+ '\n\n' + newWorld.join('\n') + '\n\n');




